﻿using ComPDFKit.PDFAnnotation;
using ComPDFKit.PDFDocument;
using ComPDFKit.PDFPage;
using ComPDFKit.PDFPage.Edit;
using ComPDFKit.Tool;
using ComPDFKit.Tool.SettingParam;
using Compdfkit_Tools.Edit;
using Compdfkit_Tools.Helper;
using ComPDFKitViewer;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using static ComPDFKit.Tool.CPDFToolManager;
using System.Windows.Media.Imaging;
using ComPDFKit.Tool.Help;
using ComPDFKit.Tool.UndoManger;

namespace Compdfkit_Tools.PDFControl
{
    public partial class ContentEditControl : UserControl, INotifyPropertyChanged
    {
        #region Property
        public PDFViewControl PdfViewControl;
        public PDFContentEditControl pdfContentEditControl = new PDFContentEditControl();
        private CPDFDisplaySettingsControl displaySettingsControl = null;

        private TextEditParam pdfTextCreateParam;
        private PDFEditParam lastPDFEditEvent = null;
        private List<TextEditParam> lastPDFEditMultiEvents = null;
        private PanelState panelState = PanelState.GetInstance();
        private KeyEventHandler KeyDownHandler;
        public event PropertyChangedEventHandler PropertyChanged;
        public ICommand CloseTabCommand;
        public ICommand ExpandPropertyPanelCommand;

        public bool CanUndo
        {
            get
            {
                try
                {
                    if (PdfViewControl != null && PdfViewControl.PDFViewTool != null)
                    {
                        CPDFViewerTool viewerTool = PdfViewControl.PDFViewTool;
                        CPDFViewer pdfViewer = viewerTool.GetCPDFViewer();
                        return pdfViewer.UndoManager.CanUndo;
                    }
                }
                catch (Exception ex)
                {

                }

                return false;
            }
        }

        public bool CanRedo
        {
            get
            {
                try
                {
                    if (PdfViewControl != null && PdfViewControl.PDFViewTool != null)
                    {
                        CPDFViewerTool viewerTool = PdfViewControl.PDFViewTool;
                        CPDFViewer pdfViewer = viewerTool.GetCPDFViewer();
                        return pdfViewer.UndoManager.CanRedo;
                    }
                }
                catch (Exception ex)
                {

                }

                return false;
            }
        }

        private bool CanSave
        {
            get
            {
                try
                {
                    if (PdfViewControl != null && PdfViewControl.PDFViewTool != null)
                    {
                        CPDFViewerTool viewerTool = PdfViewControl.PDFViewTool;
                        CPDFViewer pdfViewer = viewerTool.GetCPDFViewer();
                        return (pdfViewer.UndoManager.CanUndo | pdfViewer.UndoManager.CanRedo);
                    }
                }
                catch (Exception ex)
                {

                }

                return false;
            }
        }

        public event EventHandler<bool> OnCanSaveChanged;
        public event EventHandler OnAnnotEditHandler;

        #endregion

        public ContentEditControl()
        {
            InitializeComponent();
            panelState.PropertyChanged -= PanelState_PropertyChanged;
            panelState.PropertyChanged += PanelState_PropertyChanged;
        }

        private void PanelState_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(PanelState.IsLeftPanelExpand))
            {
                ExpandLeftPanel(panelState.IsLeftPanelExpand);
            }
            else if (e.PropertyName == nameof(PanelState.RightPanel))
            {
                if (panelState.RightPanel == PanelState.RightPanelState.PropertyPanel)
                {
                    ExpandRightPropertyPanel(pdfContentEditControl, Visibility.Visible);
                }
                else if (panelState.RightPanel == PanelState.RightPanelState.ViewSettings)
                {
                    ExpandRightPropertyPanel(displaySettingsControl, Visibility.Visible);
                }
                else
                {
                    ExpandRightPropertyPanel(null, Visibility.Collapsed);
                }
            }
        }

        public void ClearPDFEditState(ToggleButton ignoreBtn = null)
        {
            List<ToggleButton> clearBtnList = new List<ToggleButton>()
            {
                PDFTextEditButton,
                PDFImageEditButton
            };

            foreach (ToggleButton item in clearBtnList)
            {
                if (ignoreBtn == item)
                {
                    continue;
                }
                item.IsChecked = false;
            }
            PdfViewControl.PDFToolManager.SetCreateContentEditType(CPDFEditType.None);
            PdfViewControl.PDFViewTool.GetCPDFViewer().SetIsVisibleCustomMouse(false);
            PdfViewControl.PDFViewTool.GetCPDFViewer().SetIsShowStampMouse(false);
        }

        private void PDFTextEditButton_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton senderBtn = sender as ToggleButton;
            if (senderBtn != null && PdfViewControl != null)
            {
                ClearPDFEditState(senderBtn);
                if (senderBtn.IsChecked == true)
                {
                    TextEditParam textEditParam = new TextEditParam();
                    textEditParam.EditType = CPDFEditType.EditText;
                    textEditParam.IsBold = false;
                    textEditParam.IsItalic = false;
                    textEditParam.FontSize = 14;
                    textEditParam.FontName = "Helvetica";
                    textEditParam.FontColor = new byte[] { 0, 0, 0 };
                    textEditParam.EditIndex = -1;
                    textEditParam.TextAlign = TextAlignType.AlignLeft;
                    textEditParam.Transparency = 255;
                    pdfContentEditControl.SetPDFTextEditData(textEditParam);
                    DefaultSettingParam defaultSettingParam = PdfViewControl.PDFViewTool.GetDefaultSettingParam();
                    defaultSettingParam.SetPDFEditParamm(textEditParam);
                    panelState.RightPanel = PanelState.RightPanelState.PropertyPanel;
                    PdfViewControl.PDFToolManager.SetCreateContentEditType(CPDFEditType.EditText);
                }
                else
                {

                }
                //    PDFEditEvent createParam = new PDFEditEvent();
                //    createParam.EditType = CPDFEditType.EditText;
                //    createParam.IsBold = false;
                //    createParam.IsItalic = false;
                //    createParam.FontSize = 14;
                //    createParam.FontName = "Helvetica";
                //    createParam.FontColor = Colors.Black;
                //    createParam.TextAlign = TextAlignType.AlignLeft;
                //    createParam.Transparency = 255;

                //    if (PdfViewControl.PDFView != null && PdfViewControl.PDFView.Document != null)
                //    {
                //        CPDFDocument pdfDoc = PdfViewControl.PDFView.Document;
                //        PdfViewControl.PDFView.ToolManager.EnableClickCreate = true;
                //        PdfViewControl.PDFView.ToolManager.ClickCreateWidth = 100;
                //        if (pdfDoc.PageCount > 0)
                //        {
                //            CPDFPage pdfPage = pdfDoc.PageAtIndex(0);
                //            CPDFEditPage editPage = pdfPage.GetEditPage();
                //            editPage.BeginEdit(CPDFEditType.EditText);
                //            createParam.SystemFontNameList.AddRange(editPage.GetFontList());
                //            editPage.EndEdit();
                //        }
                //    }

                //    PdfViewControl.PDFView?.SetPDFEditType(CPDFEditType.EditText);
                //    PdfViewControl.PDFView?.SetPDFEditCreateType(CPDFEditType.EditText);
                //    PdfViewControl.PDFView?.SetMouseMode(MouseModes.PDFEdit);
                //    PdfViewControl.PDFView?.ReloadDocument();

                //    PdfViewControl.PDFView?.SetPDFEditParam(createParam);
                //    pdfContentEditControl.SetPDFTextEditData(createParam);
                //    panelState.RightPanel = PanelState.RightPanelState.PropertyPanel;
                //    pdfTextCreateParam = createParam;
                //}
                //else
                //{
                //    PdfViewControl.PDFView?.SetPDFEditCreateType(CPDFEditType.None);
                //    PdfViewControl.PDFView?.SetPDFEditType(CPDFEditType.EditImage | CPDFEditType.EditText);
                //    PdfViewControl.PDFView?.SetMouseMode(MouseModes.PDFEdit);
                //    PdfViewControl.PDFView?.ReloadDocument();
                //    pdfContentEditControl.ClearContentControl();
                //    panelState.RightPanel = PanelState.RightPanelState.None;
                //}
            }

        }

        private void PDFImageEditButton_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton senderBtn = sender as ToggleButton;
            if (senderBtn != null && PdfViewControl != null)
            {
                panelState.RightPanel = PanelState.RightPanelState.None;
                senderBtn.IsChecked = false;
                OpenFileDialog openFileDialog = new OpenFileDialog();
                openFileDialog.Filter = "Image Files(*.jpg;*.jpeg;*.png;*.bmp)|*.jpg;*.jpeg;*.png;*.bmp;";
                if (openFileDialog.ShowDialog() == true)
                {
                    ClearPDFEditState(senderBtn);
                    try
                    {
                        bool SetImage = PdfViewControl.PDFViewTool.GetCPDFViewer().SetStampMouseImage(openFileDialog.FileName);
                        PdfViewControl.PDFToolManager.SetCreateImagePath(openFileDialog.FileName);
                        PdfViewControl.PDFViewTool.GetCPDFViewer().SetIsVisibleCustomMouse(SetImage);
                        PdfViewControl.PDFViewTool.GetCPDFViewer().SetIsShowStampMouse(SetImage);
                        PdfViewControl.PDFToolManager.SetCreateContentEditType(CPDFEditType.EditImage);
                    }
                    catch
                    {
                    }
                    //if (openFileDialog.ShowDialog() == true)
                    //{
                    //    ClearPDFEditState(senderBtn);
                    //    PdfViewControl.PDFView?.ClearSelectPDFEdit();
                    //    PdfViewControl.PDFView?.SetPDFEditType(CPDFEditType.EditImage | CPDFEditType.EditText);
                    //    PdfViewControl.PDFView?.SetMouseMode(MouseModes.PDFEdit);
                    //    PdfViewControl.PDFView?.ReloadDocument();

                    //    PdfViewControl.PDFView?.SetPDFEditCreateType(CPDFEditType.EditImage);
                    //    PdfViewControl.PDFView?.AddPDFEditImage(openFileDialog.FileName); 
                    //}
                }
            }
        }

        public void ExpandLeftPanel(bool isExpand)
        {
            BotaContainer.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            Splitter.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            if (isExpand)
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(320);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(15);
            }
            else
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(0);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(0);
            }
        }

        private void UndoBtn_Click(object sender, RoutedEventArgs e)
        {
            if (PdfViewControl != null && PdfViewControl.PDFViewTool != null)
            {
                PdfViewControl.PDFViewTool.GetCPDFViewer()?.UndoManager?.Undo();
                //PdfViewControl.PDFToolManager.ClearSelect();
                PdfViewControl.PDFViewTool.GetCPDFViewer().UpDateRenderFrame();
            }
        }

        private void RedoBtn_Click(object sender, RoutedEventArgs e)
        {
            if (PdfViewControl != null && PdfViewControl.PDFViewTool != null)
            {
                PdfViewControl.PDFViewTool.GetCPDFViewer()?.UndoManager?.Redo();
                //PdfViewControl.PDFToolManager.ClearSelect();
                PdfViewControl.PDFViewTool.GetCPDFViewer().UpDateRenderFrame();
            }
        }

        public void SetViewSettings(Visibility visibility, CPDFDisplaySettingsControl displaySettingsControl = null)
        {
            this.PropertyContainer.Child = displaySettingsControl;
            this.PropertyContainer.Visibility = visibility;
        }

        public void SetBOTAContainer(CPDFBOTABarControl botaControl)
        {
            this.BotaContainer.Child = botaControl;
        }

        public void SetDisplaySettingsControl(CPDFDisplaySettingsControl displaySettingsControl)
        {
            this.displaySettingsControl = displaySettingsControl;
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {

        }

        private void UserControl_UnLoaded(object sender, RoutedEventArgs e)
        {

        }

        public void ExpandRightPropertyPanel(UIElement propertytPanel, Visibility visible)
        {
            PropertyContainer.Width = 260;
            PropertyContainer.Child = propertytPanel;
            PropertyContainer.Visibility = visible;
        }

        public void InitWithPDFViewer(PDFViewControl view)
        {
            PdfViewControl.PDFViewTool.GetCPDFViewer().UndoManager.PropertyChanged -= UndoManager_PropertyChanged;
            PdfViewControl.PDFViewTool.GetCPDFViewer().UndoManager.PropertyChanged += UndoManager_PropertyChanged;
            PdfViewControl.MouseLeftButtonDownHandler -= PDFToolManager_MouseLeftButtonDownHandler;
            PdfViewControl.MouseLeftButtonDownHandler += PDFToolManager_MouseLeftButtonDownHandler;
            PdfViewControl.MouseLeftButtonUpHandler -= PDFToolManager_MouseLeftButtonUpHandler;
            PdfViewControl.MouseLeftButtonUpHandler += PDFToolManager_MouseLeftButtonUpHandler;
            PdfViewControl.MouseRightButtonDownHandler -= PdfViewControl_MouseRightButtonDownHandler;
            PdfViewControl.MouseRightButtonDownHandler += PdfViewControl_MouseRightButtonDownHandler;
            PdfViewControl = view;
            PDFGrid.Child = PdfViewControl;
            FloatPageTool.InitWithPDFViewer(view);
            pdfContentEditControl.InitWithPDFViewer(view);
            DataContext = this;
            //if(pdfViewer!=null)
            //{
            //    pdfViewer.EnableMultiSelectEdit = true;
            //    pdfViewer.PDFEditMultiActiveHandler -= PDFEditMultiActiveHandler;
            //    pdfViewer.PDFEditMultiActiveHandler += PDFEditMultiActiveHandler;
            //    if (KeyDownHandler != null)
            //    {
            //        pdfViewer.RemoveHandler(KeyDownEvent, KeyDownHandler);
            //    }
            //    KeyDownHandler = new KeyEventHandler(PDFView_KeyDown);
            //    pdfViewer.AddHandler(KeyDownEvent, KeyDownHandler, false, true);
            //}
        }

        private void PDFEditEmptyPanel()
        {
            PropertyContainer.Child = pdfContentEditControl;

            if (pdfTextCreateParam != null && PdfViewControl != null && PdfViewControl.PDFView != null)
            {
                if (PdfViewControl.PDFToolManager.GetCreateContentEditType() == CPDFEditType.EditText)
                {
                    pdfContentEditControl.SetPDFTextEditData(pdfTextCreateParam);
                }
                else if (PdfViewControl.PDFToolManager.GetCreateContentEditType() == CPDFEditType.None)
                {
                    pdfContentEditControl.ClearContentControl();

                }
            }
            else
            {
                pdfContentEditControl.ClearContentControl();
            }
        }

        //private void PDFEditMultiActiveHandler(object sender, List<PDFEditEvent> e)
        //{
        //    lastPDFEditEvent = null;
        //    lastPDFEditMultiEvents = e;
        //    if(e==null)
        //    {
        //        PDFEditEmptyPanel();
        //        return;
        //    }

        //    if(e.Count>1)
        //    {
        //       List<CPDFEditType> editList= e.AsEnumerable().Select(x=>x.EditType).Distinct().ToList();
               
        //        if(editList.Count>1)
        //        {
        //            PDFEditEmptyPanel();
        //            return;
        //        }

        //        if (editList[0]==CPDFEditType.EditText)
        //        {
        //            pdfContentEditControl.SetPDFTextMultiEditData(e);
        //            return;
        //        }

        //        if (editList[0]==CPDFEditType.EditImage)
        //        {
        //            UIElement pageView = sender as UIElement;
        //            if (pageView != null)
        //            {
        //                pageView.MouseLeftButtonUp += PageView_MouseLeftButtonUp;
        //            }
        //            pdfContentEditControl.SetPDFImageMultiEditData(e);
        //        }
        //    }
        //}

        //public void PDFView_KeyDown(object sender, KeyEventArgs e)
        //{
        //    if (PdfViewControl.PDFView.MouseMode != MouseModes.PDFEdit)
        //    {
        //        return;
        //    }

        //    if (Keyboard.Modifiers == ModifierKeys.Control)
        //    {
        //        if (e.Key == Key.Left)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypePreWord, false);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Right)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeNextWord, false);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Up)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeSectionBegin, false);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Down)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeSectionEnd, false);
        //            e.Handled = true;
        //        }
        //    }

        //    if (Keyboard.Modifiers == ModifierKeys.Shift)
        //    {
        //        if (e.Key == Key.Left)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypePreCharPlace, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Right)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeNextCharPlace, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Up)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeUpCharPlace, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Down)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeDownCharPlace, true);
        //            e.Handled = true;
        //        }
        //    }

        //    if (Keyboard.Modifiers == ModifierKeys.Alt)
        //    {
        //        if (e.SystemKey == Key.Up)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLocationLineBegin, false);
        //            e.Handled = true;
        //        }

        //        if (e.SystemKey == Key.Down)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeLineEnd, false);
        //            e.Handled = true;
        //        }
        //    }

        //    if (Keyboard.Modifiers == ModifierKeys.None)
        //    {
        //        if (e.Key == Key.Left)
        //        {
        //            PdfViewControl.PDFView.MoveEditArea(new Point(-5, 0));
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Right)
        //        {
        //            PdfViewControl.PDFView.MoveEditArea(new Point(5, 0));
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Up)
        //        {
        //            PdfViewControl.PDFView.MoveEditArea(new Point(0, -5));
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Down)
        //        {
        //            PdfViewControl.PDFView.MoveEditArea(new Point(0, 5));
        //            e.Handled = true;
        //        }
        //    }

        //    if (Keyboard.Modifiers == (ModifierKeys.Control | ModifierKeys.Shift))
        //    {
        //        if (e.Key == Key.Left)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypePreWord, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Right)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeNextWord, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Up)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeSectionBegin, true);
        //            e.Handled = true;
        //        }

        //        if (e.Key == Key.Down)
        //        {
        //            PdfViewControl.PDFView.JumpEditingLoction(CEditingLocation.CEditingLoadTypeSectionEnd, true);
        //            e.Handled = true;
        //        }
        //    }
        //}
        private void PdfViewControl_MouseRightButtonDownHandler(object sender, MouseEventObject e)
        {
            ContextMenu ContextMenu = PdfViewControl.GetRightMenu();
            if (ContextMenu == null)
            {
                ContextMenu = new ContextMenu();
            }
            switch (e.hitTestType)
            {
                case MouseHitTestType.kTextEdit:
                case MouseHitTestType.kImageEdit:
                    CreateEditMenu(sender, ref ContextMenu);
                    break;
                default:
                    ContextMenu.Items.Add(new MenuItem() { Header = "Paste", Command = ApplicationCommands.Paste, CommandTarget = (UIElement)sender });
                    break;
            }
            PdfViewControl.SetRightMenu(ContextMenu);
        }
        private void CreateEditMenu(object sender, ref ContextMenu menu)
        {
            menu.Items.Add(new MenuItem() { Header = "Delete", Command = ApplicationCommands.Delete, CommandTarget = (UIElement)sender });
            menu.Items.Add(new MenuItem() { Header = "Copy", Command = ApplicationCommands.Copy, CommandTarget = (UIElement)sender });
            menu.Items.Add(new MenuItem() { Header = "Cut", Command = ApplicationCommands.Cut, CommandTarget = (UIElement)sender });
            menu.Items.Add(new MenuItem() { Header = "Paste", Command = ApplicationCommands.Paste, CommandTarget = (UIElement)sender });
        }
        private void PDFToolManager_MouseLeftButtonUpHandler(object sender, MouseEventObject e)
        {
            if (e.IsCreate)
            {
                PdfViewControl.SetIsShowStampMouse(false);
            }
        }

        private void PDFToolManager_MouseLeftButtonDownHandler(object sender, MouseEventObject e)
        {

            if (PdfViewControl.PDFToolManager.GetToolType() != ToolType.ContentEdit)
            {
                PropertyContainer.Child = pdfContentEditControl;

                //if (pdfTextCreateParam != null && PdfViewControl != null && PdfViewControl.PDFView != null)
                //{
                //    if (PdfViewControl.PDFView.GetPDFEditCreateType() == CPDFEditType.EditText)
                //    {
                //        pdfContentEditControl.SetPDFTextEditData(pdfTextCreateParam);
                //    }
                //    else if (PdfViewControl.PDFView.GetPDFEditCreateType() == CPDFEditType.None)
                //    {
                //        pdfContentEditControl.ClearContentControl();
                //    }
                //}
                //else
                //{
                //    pdfContentEditControl.ClearContentControl();
                //}
                return;
            }

            int PageIndex = -1;
            CPDFEditArea editAreaArea = PdfViewControl.PDFToolManager.GetSelectedEditAreaObject(ref PageIndex);
            if (editAreaArea == null)
            {
                return;
            }
            else
            {
                if (editAreaArea.Type == CPDFEditType.EditText)
                {
                    PDFEditParam pDFEditParam = ParamConverter.CPDFDataConverterToPDFEitParam(PdfViewControl.PDFToolManager.GetDocument(), editAreaArea, PageIndex);
                    pdfContentEditControl.SetPDFTextEditData((TextEditParam)pDFEditParam, true);
                    PropertyContainer.Child = pdfContentEditControl;
                }

                else if (editAreaArea.Type == CPDFEditType.EditImage && PdfViewControl != null)
                {
                    UIElement pageView = sender as UIElement;
                    if (pageView != null)
                    {
                        pageView.MouseLeftButtonUp -= PageView_MouseLeftButtonUp;
                        pageView.MouseLeftButtonUp += PageView_MouseLeftButtonUp;
                    }
                    PDFEditParam pDFEditParam = ParamConverter.CPDFDataConverterToPDFEitParam(PdfViewControl.PDFToolManager.GetDocument(), editAreaArea, PageIndex);
                    pdfContentEditControl.SetPDFImageEditData((ImageEditParam)pDFEditParam);
                    PropertyContainer.Child = pdfContentEditControl;
                }
                panelState.RightPanel = PanelState.RightPanelState.PropertyPanel;
            }
        }

        private void PdfViewControl_SplitPDFViewToolCreated(object sender, EventArgs e)
        {
            PdfViewControl.PDFViewTool.GetCPDFViewer().UndoManager.PropertyChanged -= UndoManager_PropertyChanged;
            PdfViewControl.PDFViewTool.GetCPDFViewer().UndoManager.PropertyChanged += UndoManager_PropertyChanged;
        }

        /// <summary>
        /// Text and Image Selected Event
        /// </summary>
        private void PDFView_PDFEditActiveHandler(object sender, PDFEditParam e)
        {
            lastPDFEditEvent = e;

            if (e == null)
            {
                PropertyContainer.Child = pdfContentEditControl;

                //if (pdfTextCreateParam != null && PdfViewControl != null && PdfViewControl.PDFView != null)
                //{
                //    if (PdfViewControl.PDFView.GetPDFEditCreateType() == CPDFEditType.EditText)
                //    {
                //        pdfContentEditControl.SetPDFTextEditData(pdfTextCreateParam);
                //    }
                //    else if (PdfViewControl.PDFView.GetPDFEditCreateType() == CPDFEditType.None)
                //    {
                //        pdfContentEditControl.ClearContentControl();
                //    }
                //}
                //else
                //{
                //    pdfContentEditControl.ClearContentControl();
                //}
                return;
            }

            if (e.EditType == CPDFEditType.EditText)
            {
                pdfContentEditControl.SetPDFTextEditData((TextEditParam)e, true);
                return;
            }

            if (e.EditType == CPDFEditType.EditImage && PdfViewControl != null)
            {
                UIElement pageView = sender as UIElement;
                if (pageView != null)
                {
                    pageView.MouseLeftButtonUp -= PageView_MouseLeftButtonUp;
                    pageView.MouseLeftButtonUp += PageView_MouseLeftButtonUp;
                }
                pdfContentEditControl.SetPDFImageEditData((ImageEditParam)e);
                return;
            }
        }

        private void PageView_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            UIElement pageView = sender as UIElement;
            if (pageView != null)
            {
                pageView.MouseLeftButtonUp -= PageView_MouseLeftButtonUp;
            }
            if (lastPDFEditEvent != null && lastPDFEditEvent.EditType == CPDFEditType.EditImage)
            {
                pdfContentEditControl.SetPDFImageEditData((ImageEditParam)lastPDFEditEvent);
            }
        }

        public void ClearViewerControl()
        {
            PDFGrid.Child = null;
            BotaContainer.Child = null;
            PropertyContainer.Child = null;
            displaySettingsControl = null;
        }

        #region Property changed
        protected void OnPropertyChanged([CallerMemberName] string name = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }


        public void UndoManager_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            OnPropertyChanged(e.PropertyName);
            if (e.PropertyName == "CanSave")
            {
                OnCanSaveChanged?.Invoke(this, CanSave);
            }
        }
        #endregion

        private void CommandBinding_Executed_Undo(object sender, ExecutedRoutedEventArgs e)
        {
            if (PdfViewControl != null && PdfViewControl.PDFViewTool != null && CanUndo)
            {
                PdfViewControl.PDFViewTool.GetCPDFViewer()?.UndoManager?.Undo();
            }
        }

        private void CommandBinding_Executed_Redo(object sender, ExecutedRoutedEventArgs e)
        {
            if (PdfViewControl != null && PdfViewControl.PDFViewTool != null && CanRedo)
            {
                PdfViewControl.PDFViewTool.GetCPDFViewer()?.UndoManager?.Redo();
            }
        }
    }
}
